@@ -5,6 +5,7 @@
 #include <stdint.h>
 #include <inttypes.h>
 #include <stdio.h>
+#include <limits.h>
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -12,8 +13,90 @@
 #include <unistd.h>
 #include <err.h>
 #include <sys/time.h>
+#include <argp.h>
 
 #include "utils.h"
+#include "version.h"
+
+/* Argp's global variables. */
+const char *argp_program_version = "F3 Read " F3_STR_VERSION;
+
+/* Arguments. */
+static char adoc[] = "<PATH>";
+
+static char doc[] = "F3 Read -- validate .h2w files to test "
+	"the real capacity of the drive";
+
+static struct argp_option options[] = {
+	{"start-at",		's',	"NUM",		0,
+		"First NUM.h2w file to be read",			1},
+	{"end-at",		'e',	"NUM",		0,
+		"Last NUM.h2w file to be read",				0},
+	{"show-progress",	'p',	"NUM",		0,
+		"Show progress if NUM is not zero",			0},
+	{ 0 }
+};
+
+struct args {
+	long        start_at;
+	long        end_at;
+	int	    show_progress;
+	const char  *dev_path;
+};
+
+static error_t parse_opt(int key, char *arg, struct argp_state *state)
+{
+	struct args *args = state->input;
+	long l;
+
+	switch (key) {
+	case 's':
+		l = arg_to_long(state, arg);
+		if (l <= 0)
+			argp_error(state,
+				"NUM must be greater than zero");
+		args->start_at = l - 1;
+		break;
+
+	case 'e':
+		l = arg_to_long(state, arg);
+		if (l <= 0)
+			argp_error(state,
+				"NUM must be greater than zero");
+		args->end_at = l - 1;
+		break;
+
+	case 'p':
+		args->show_progress = !!arg_to_long(state, arg);
+		break;
+
+	case ARGP_KEY_INIT:
+		args->dev_path = NULL;
+		break;
+
+	case ARGP_KEY_ARG:
+		if (args->dev_path)
+			argp_error(state,
+				"Wrong number of arguments; only one is allowed");
+		args->dev_path = arg;
+		break;
+
+	case ARGP_KEY_END:
+		if (!args->dev_path)
+			argp_error(state,
+				"The disk path was not specified");
+		if (args->start_at > args->end_at)
+			argp_error(state,
+				"Option --start-at must be less or equal to option --end-at");
+		break;
+
+	default:
+		return ARGP_ERR_UNKNOWN;
+	}
+	return 0;
+}
+
+static struct argp argp = {options, parse_opt, adoc, doc, NULL, NULL, NULL};
 
 static inline void update_dt(struct timeval *dt, const struct timeval *t1,
 	const struct timeval *t2)
@@ -42,16 +125,16 @@ static void validate_file(const char *path, int number,
 {
 	char *full_fn;
 	const char *filename;
-	uint8_t sector[SECTOR_SIZE], *p, *ptr_end;
+	const int num_int64 = SECTOR_SIZE >> 3;
+	uint64_t sector[num_int64];
 	FILE *f;
 	int fd;
-	int offset_match, error_count;
 	size_t sectors_read;
-	uint64_t offset, expected_offset;
+	uint64_t expected_offset;
 	int final_errno;
 	struct timeval t1, t2;
 	/* Progress time. */
-	struct timeval pt1 = { .tv_sec = -1000, .tv_usec = 0 };
+	struct timeval pt1;
 
 	*ptr_ok = *ptr_corrupted = *ptr_changed = *ptr_overwritten = 0;
 
@@ -81,35 +164,29 @@ static void validate_file(const char *path, int number,
 
 	/* Obtain initial time. */
 	assert(!gettimeofday(&t1, NULL));
+	pt1 = t1;
+	pt1.tv_sec -= 1000;
 	/* Help the kernel to help us. */
 	assert(!posix_fadvise(fd, 0, 0, POSIX_FADV_SEQUENTIAL));
 
-	ptr_end = sector + SECTOR_SIZE;
 	sectors_read = fread(sector, SECTOR_SIZE, 1, f);
 	final_errno = errno;
 	expected_offset = (uint64_t)number * GIGABYTES;
 	while (sectors_read > 0) {
 		uint64_t rn;
+		int error_count, i;
 
 		assert(sectors_read == 1);
-		offset = *((uint64_t *) sector);
-		offset_match = offset == expected_offset;
 
-		rn = offset;
-		p = sector + sizeof(offset);
+		rn = sector[0];
 		error_count = 0;
-		for (; error_count <= TOLERANCE && p < ptr_end;
-			p += sizeof(rn)) {
+		for (i = 1; error_count <= TOLERANCE && i < num_int64; i++) {
 			rn = random_number(rn);
-			if (rn != *((__typeof__(rn) *) p))
+			if (rn != sector[i])
 				error_count++;
 		}
 
-		sectors_read = fread(sector, SECTOR_SIZE, 1, f);
-		final_errno = errno;
-		expected_offset += SECTOR_SIZE;
-
-		if (offset_match) {
+		if (expected_offset == sector[0]) {
 			if (error_count == 0)
 				(*ptr_ok)++;
 			else if (error_count <= TOLERANCE)
@@ -121,6 +198,10 @@ static void validate_file(const char *path, int number,
 		else
 			(*ptr_corrupted)++;
 
+		sectors_read = fread(sector, SECTOR_SIZE, 1, f);
+		final_errno = errno;
+		expected_offset += SECTOR_SIZE;
+
 		if (progress) {
 			struct timeval pt2;
 			assert(!gettimeofday(&pt2, NULL));
@@ -173,7 +254,9 @@ static void iterate_files(const char *path, const long *files,
 	const char *unit;
 	int and_read_all = 1;
 	int or_missing_file = 0;
-	int number = start_at;
+	long number = start_at;
+
+	UNUSED(end_at);
 
 	tot_ok = tot_corrupted = tot_changed = tot_overwritten = tot_size = 0;
 	printf("                  SECTORS "
@@ -220,7 +303,7 @@ static void iterate_files(const char *path, const long *files,
 	report("\t     Overwritten:", tot_overwritten);
 	if (or_missing_file)
 		printf("WARNING: Not all F3 files in the range %li to %li are available\n",
-			start_at + 1, end_at + 1);
+			start_at + 1, number);
 	if (!and_read_all)
 		printf("WARNING: Not all data was read due to I/O error(s)\n");
 
@@ -232,20 +315,24 @@ static void iterate_files(const char *path, const long *files,
 
 int main(int argc, char **argv)
 {
-	long start_at, end_at;
-	const char *path;
 	const long *files;
-	int progress;
-	int rc;
 
-	rc = parse_args("read", argc, argv, &start_at, &end_at, &path);
-	if (rc)
-		return rc;
+	struct args args = {
+		/* Defaults. */
+		.start_at	= 0,
+		.end_at		= LONG_MAX - 1,
+		/* If stdout isn't a terminal, supress progress. */
+		.show_progress	= isatty(STDOUT_FILENO),
+	};
 
-	files = ls_my_files(path, start_at, end_at);
-	/* If stdout isn't a terminal, supress progress. */
-	progress = isatty(STDOUT_FILENO);
-	iterate_files(path, files, start_at, end_at, progress);
+	/* Read parameters. */
+	argp_parse(&argp, argc, argv, 0, NULL, &args);
+	print_header(stdout, "read");
+
+	files = ls_my_files(args.dev_path, args.start_at, args.end_at);
+
+	iterate_files(args.dev_path, files, args.start_at, args.end_at,
+		args.show_progress);
 	free((void *)files);
 	return 0;
 }
